//===========================================================================
//  $Name: arts++-1-1-a13 $
//  $Id: ArtsBgp4DPAttribute.hh,v 1.2 2004/04/21 23:51:25 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSBGP4DPATTRIBUTE_HH_
#define _ARTSBGP4DPATTRIBUTE_HH_

//---------------------------------------------------------------------------
//  class ArtsBgp4DPAttribute
//---------------------------------------------------------------------------
//!  This class abstracts a BGP4 DPA (Destination Preference Attribute)
//!  value.
//---------------------------------------------------------------------------
class ArtsBgp4DPAttribute
{
public:
  //--------------------------------------------------------------------------
  //                          ArtsBgp4DPAttribute()
  //..........................................................................
  //!  constructor
  //--------------------------------------------------------------------------
  ArtsBgp4DPAttribute();
  
  //--------------------------------------------------------------------------
  //                         ~ArtsBgp4DPAttribute()
  //..........................................................................
  //!  destructor
  //--------------------------------------------------------------------------
  ~ArtsBgp4DPAttribute();

  //-------------------------------------------------------------------------
  //                           uint16_t AS() const
  //.........................................................................
  //!  Returns the AS number in the DPA attribute.
  //-------------------------------------------------------------------------
  uint16_t AS() const;
  
  //-------------------------------------------------------------------------
  //                        uint16_t AS(uint16_t as)
  //.........................................................................
  //!  Sets and returns the AS number in the DPA attribute.
  //-------------------------------------------------------------------------
  uint16_t AS(uint16_t as);
  
  //-------------------------------------------------------------------------
  //                         uint32_t Value() const
  //.........................................................................
  //!  Returns the value in the DPA attribute.
  //-------------------------------------------------------------------------
  uint32_t Value() const;
  
  //-------------------------------------------------------------------------
  //                     uint32_t Value(uint32_t value)
  //.........................................................................
  //!  Sets and returns the value in the DPA attribute.
  //-------------------------------------------------------------------------
  uint32_t Value(uint32_t value);

  //-------------------------------------------------------------------------
  // ArtsBgp4DPAttribute & operator = (const ArtsBgp4DPAttribute & attribute)
  //.........................................................................
  //!  Overloaded '=' operator for assigning the value of an
  //!  ArtsBgp4DPAttribute from another ArtsBgp4DPAttribute.
  //-------------------------------------------------------------------------
  ArtsBgp4DPAttribute & operator = (const ArtsBgp4DPAttribute & attribute);

  //--------------------------------------------------------------------------
  //            std::istream & read(std::istream & is, uint8_t version = 0)
  //..........................................................................
  //!  Reads the contents of an ArtsBgp4DPAttribute from an istream.
  //!  Returns the istream.
  //--------------------------------------------------------------------------
  std::istream & read(std::istream & is, uint8_t version = 0);

  //--------------------------------------------------------------------------
  //                  int read(int fd, uint8_t version = 0)
  //..........................................................................
  //!  Reads the contents of an ArtsBgp4DPAttribute from a file descriptor.
  //!  Returns the number of bytes written on success, -1 on failure.
  //--------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);

  //--------------------------------------------------------------------------
  //        std::ostream & write(std::ostream & os, uint8_t version = 0) const
  //..........................................................................
  //!  Writes the contents of an ArtsBgp4DPAttribute to an ostream.
  //!  Returns the ostream.
  //--------------------------------------------------------------------------
  std::ostream & write(std::ostream & os, uint8_t version = 0) const;

  //--------------------------------------------------------------------------
  //              int write(int fd, uint8_t version = 0) const
  //..........................................................................
  //!  Writes the contents of an ArtsBgp4DPAttribute to a file descriptor.
  //!  Returns the number of bytes written on success, -1 on failure.
  //--------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0) const;

  //--------------------------------------------------------------------------
  //                uint32_t Length(uint8_t version = 0) const               
  //..........................................................................
  //!  Returns the number of bytes required to store the
  //!  ArtsBgp4DPAttribute on disk.
  //--------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const
  {
    return(sizeof(this->_as) + sizeof(this->_value));
  }

  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                  static uint32_t NumObjects()
    //........................................................................
    //  
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG

private:
  uint16_t   _as;
  uint32_t   _value;

  static uint32_t  _numObjects;
};

#endif  // _ARTSBGP4DPATTRIBUTE_HH_
